#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>

#define unhex_table(h) ((h >= 'A' && h <= 'F') ? 10 + h - 'A' : h - '0')

////////////////////////////////////////////////////////////////////////////
// Decodes ascii hex to binary data.  
void DecodeHex(void *dst, size_t dst_len, const char *src, size_t src_len)
{
  size_t i;
  int sl, sh;

  assert(dst_len >= src_len / 2);

  for (i = 0; i < dst_len; i++)
  {
    sh = unhex_table(src[2 * i + 0]);
    sl = unhex_table(src[2 * i + 1]);
    ((unsigned char*) dst)[i] = (sh << 4) | sl;
  }
  
  return;
}

#define MAX_IMG_SIZE	(1024*1024*3)

int
processLogFile(const char *name)
{
    FILE *logFile;
    char linebuf[MAX_IMG_SIZE+1024];
    int n = 0;

    logFile = fopen(name, "r");

    if(!logFile) {
	printf("error opening file: %s\n", name);
	return -1;
    }

    while(fgets(linebuf, sizeof(linebuf), logFile)) {
	double stime;
	char host[255];
	int port;
	char device[255];
	int device_index;
	double time;

	int fields;

	fields = sscanf(linebuf, "%lf %s %d %s %d %lf",
			&stime, host, &port, device, &device_index, &time);
	if ((fields == 6) && !strcmp(device, "camera")) {

	    int width, height, bpp, format, compression, size;

	    fields = sscanf(linebuf, "%lf %s %d %s %d %lf %d %d %d %d %d %d",
			&stime, host, &port, device, &device_index, &time,
			&width, &height, &bpp, &format, &compression,
			&size);

	    printf("%dx%d = %d\n", width, height, size);

	    char *imgAsciiData = linebuf;
	    int i;

	    for(i = 0; i < 12; i++) {
		imgAsciiData = index(imgAsciiData, ' ');
		imgAsciiData++;
	    }
	    assert( strlen(imgAsciiData)/2 == size );

	    char *imgBinaryData;
	    char frameName[255];

	    sprintf(frameName, "%s_%s_%d_%014.3f.pnm",
		    host, device, device_index, time);

	    FILE *frameFile;
	    frameFile = fopen(frameName, "w");
	    if (!frameFile) {
		perror("fopen()");
		break;
	    }

	    fprintf( frameFile, "P6\n" );
	    fprintf( frameFile, "# Generated by a herd of rabid fleas\n" );
	    fprintf( frameFile, "%d %d\n", width, height );
	    fprintf( frameFile, "255\n" ); // Max val

	    imgBinaryData = (char *) malloc(size);
	    DecodeHex(imgBinaryData, size, imgAsciiData, size);

	    
	    fwrite(imgBinaryData, 1, size, frameFile);
	    fclose(frameFile);

	    free(imgBinaryData);
	}
	n++;
    }

    fclose(logFile);

    return n;
}


int
main(int argc, char** argv)
{
    int i;

    if( argc < 2) {
        printf("USAGE: cameraunlog input_image ...\n");
	return -1;
    }

    for (i = 1; i < argc; i++) {
        printf("n = %d\n", processLogFile(argv[i]));
    }

    return 0;
}
