#include <cv.h>
#include <cxmisc.h>
#include <highgui.h>
#include <stdio.h>
#include <stdlib.h>
#include "vis_timing.h"

void
createPolarLuts(const CvArr* srcarr, CvArr* dstarr,
            CvPoint2D32f center, double radius,
	    CvMat **mapx_ptr, CvMat **mapy_ptr)
{
    CvMat srcstub, *src = (CvMat*)srcarr;
    CvMat dststub, *dst = (CvMat*)dstarr;
    CvMat *mapx, *mapy;
    CvSize ssize, dsize;

    src = cvGetMat( srcarr, &srcstub );
    dst = cvGetMat( dstarr, &dststub );
    
    if( !CV_ARE_TYPES_EQ( src, dst )) {
        printf("CV_StsUnmatchedFormats");
	exit(0);
    }

    ssize = cvGetMatSize(src);
    dsize = cvGetMatSize(dst);

    mapx = cvCreateMat( dsize.height, dsize.width, CV_32F );
    mapy = cvCreateMat( dsize.height, dsize.width, CV_32F );

    int phi, rho;
    
    for( rho = 0; rho < dsize.height; rho++ )
    {
        float* mx = (float*)(mapx->data.ptr + rho*mapx->step);
        float* my = (float*)(mapy->data.ptr + rho*mapy->step);

        double r = radius - rho*radius/dsize.height;

        for( phi = 0; phi < dsize.width; phi++ )
        {
            double cp = cos(phi*2*CV_PI/dsize.width);
            double sp = sin(phi*2*CV_PI/dsize.width);
            double x = r*cp + center.x;
            double y = r*sp + center.y;

            mx[phi] = (float)x;
            my[phi] = (float)y;
        }
    }
    
    *mapx_ptr = mapx;
    *mapy_ptr = mapy;
}

int main(int argc, char** argv)
{
    IplImage* src;

    if( argc != 3) {
        printf("USAGE: omniproject input_image output_image\n");
	return -1;
    }

    CvPoint2D32f cc; 
    double radius;

    //protonek
    cc.x = 416.8986; cc.y = 290.2421; radius = 239.2513;

    src = cvLoadImage(argv[1],1);

    IplImage* dst = cvCreateImage( cvSize(lrint(2*CV_PI*radius), lrint(radius)), 8, 3 );

    CvMat *mapx, *mapy;
    createPolarLuts(src, dst, cc, radius, &mapx, &mapy);

    struct timeval tv;
    double t1;

    t1 = REALTIME;

    /*
     # CV_INTER_NN - nearest-neigbor interpolation,
     # CV_INTER_LINEAR - bilinear interpolation (used by default)
     # CV_INTER_AREA - resampling using pixel area relation. It is preferred method for image decimation that gives moire-free results. In case of zooming it is similar to CV_INTER_NN method.
     # CV_INTER_CUBIC - bicubic interpolation.
     */
    cvRemap(src, dst, mapx, mapy, CV_INTER_LINEAR);

    printf("Time taken to undist : %2.5fs\n",REALTIME-t1);

    cvReleaseMat (&mapx);
    cvReleaseMat (&mapy);
    
    cvSaveImage(argv[2], dst);
    
    /*
    cvNamedWindow( "omniproject", 1 );
    cvShowImage( "omniproject", dst );
    cvWaitKey(0);
    */

    cvReleaseImage(&src);
    cvReleaseImage(&dst);

    return 0;
}
